#include "rhfuncu.c"

/*#define HDIR_EQ_HT */
/*#define RESET_DU */
/*#define ADAMS */
/*#define TRACE_ON */


/* -------------- Dimensions -------------- */
#define DIMX 	6
#define DIMUC 	6
#define DIMP 	0

/*-------------- Global Variables -------------- */
double tsim0 = 0;
double tsim = 20;
double tf = 1;
double ht = 0.01;
double alpha = 1;
double zeta = 120;
double x0[DIMX] = { -.25, .35, 0., 0., 0., 0. };
double u0[DIMUC] = { .2, .1, .2, .3e-1, .1e-1, .2e-1 };
double hdir = 0.0001;
double rtol = 1.0e-06;
int kmax = 60;
int dv = 10;
int dstep = 1;
#define FNMHD "coxfproGUI_"


/*-------------- Global Variables Defined by User -------------- */
double M = 0.974;
double a = 0.0125;
double s = 0.0485;
double umax = 0.342;
double umin = -0.121;
double ubar = 0.1105;
double g1 = 0.28;
double g2 = 0.28;

double q[6] = {10, 15, .1, 1, 1, .1e-1};
double r[4] = {1, 1, .1e-2, .1e-2};
double sf[6] = {10, 15, .1, 1, 1, .1e-1};
double xf[6] = {0, 0, 0, 0, 0, 0};


/* -------------- dPhi/dx -------------- */
void phix(double t, double x[], double phx1[])
{
  phx1[0] = (x[0] - xf[0]) * sf[0];
phx1[1] = (x[1] - xf[1]) * sf[1];
phx1[2] = (x[2] - xf[2]) * sf[2];
phx1[3] = (x[3] - xf[3]) * sf[3];
phx1[4] = (x[4] - xf[4]) * sf[4];
phx1[5] = (x[5] - xf[5]) * sf[5];

}



/* -------------- State Equation -------------- */
void xpfunc(double t, double x[], double u[], double xprime[])
{
  double o[4];

  o[0] = u[1];
o[1] = u[0];
o[2] = (o[1] + o[0]) / M;
o[3] = x[2];
xprime[0] = x[3];
xprime[1] = x[4];
xprime[2] = x[5];
xprime[3] = cos(o[3]) * o[2];
xprime[4] = sin(o[3]) * o[2];
xprime[5] = (o[1] - o[0]) * s / a;

}



/* -------------- Costate Equation -------------- */
void lpfunc(double t, double lmd[], double linp[], double lprime[])
{
  double x[DIMX], u[DIMUC];
  double o[1];

  mmov(1,DIMX, linp, x);
  mmov(1,DIMUC, linp+DIMX, u);

  o[0] = x[2];
lprime[0] = -(x[0] - xf[0]) * q[0];
lprime[1] = -(x[1] - xf[1]) * q[1];
lprime[2] = -(o[0] - xf[2]) * q[2] + (sin(o[0]) * lmd[3] - cos(o[0]) * lmd[4]) / M * (u[0] + u[1]);
lprime[3] = -(x[3] - xf[3]) * q[3] - lmd[0];
lprime[4] = -(x[4] - xf[4]) * q[4] - lmd[1];
lprime[5] = -(x[5] - xf[5]) * q[5] - lmd[2];

}



/* -------------- Error in Optimality Condition, Hu -------------- */
void hufunc(double t, double x[], double lmd[], double u[], double hui[])
{
  double o[13];

  o[0] = 0.2e1 * u[5];
o[1] = 0.2e1 * u[4];
o[2] = x[2];
o[3] = (sin(o[2]) * lmd[4] + cos(o[2]) * lmd[3]) / M;
o[4] = lmd[5] * s / a;
o[5] = u[0];
o[6] = u[1];
o[7] = u[2];
o[8] = u[3];
o[9] = umax - ubar;
o[10] = o[5] - ubar;
o[11] = o[6] - ubar;
o[12] = o[9] * o[9];
hui[0] = o[1] * o[10] + o[5] * r[0] + o[3] + o[4];
hui[1] = o[0] * o[11] + o[6] * r[1] + o[3] - o[4];
hui[2] = -g1 / 2 + (r[2] + o[1]) * o[7];
hui[3] = -g2 / 2 + (r[3] + o[0]) * o[8];
hui[4] = o[7] * o[7] + o[10] * o[10] - o[12];
hui[5] = o[8] * o[8] + o[11] * o[11] - o[12];

}



/* -------------- Save Simulation Conditions -------------- */
void final(FILE *fp, float t_cpu, float t_s2e)
{
	int i;
	fprintf(fp, "%% Simulation Result by cohover_xfnopre_GUI.c \n");
	fprintf(fp, "Precondition = 0 \n");
	fprintf(fp, "tsim = %g\n", tsim);
	fprintf(fp, "ht = %g, dstep = %d\n", ht, dstep);
	fprintf(fp, "tf = %g, dv = %d, alpha = %g, zeta = %g \n", (float)tf, dv, (float)alpha, (float)zeta);
	fprintf(fp, "hdir = %g, rtol = %g, kmax = %d \n", (float)hdir, (float)rtol, kmax);
	fprintf(fp, "u0 = [%g", (float)u0[0] );
	for(i=1; i<DIMUC; i++)
		fprintf(fp,",%g", (float)u0[i] );
	fprintf(fp, "]\n" );
	fprintf(fp, "t_cpu = %g, t_s2e = %g  %% [sec] \n", t_cpu, t_s2e);

	fprintf(fp, "M = %g\n ", (float)M );
	fprintf(fp, "a = %g\n ", (float)a );
	fprintf(fp, "s = %g\n ", (float)s );
	fprintf(fp, "umax = %g\n ", (float)umax );
	fprintf(fp, "umin = %g\n ", (float)umin );
	fprintf(fp, "ubar = %g\n ", (float)ubar );
	fprintf(fp, "g1 = %g\n ", (float)g1 );
	fprintf(fp, "g2 = %g\n ", (float)g2 );

	fprintf(fp, "q = [%g ", (float)q[0]);
	for(i=1; i<6; i++)
		fprintf(fp, ",%g", (float)q[i] );
	fprintf(fp, "]\n" );

	fprintf(fp, "r = [%g ", (float)r[0]);
	for(i=1; i<4; i++)
		fprintf(fp, ",%g", (float)r[i] );
	fprintf(fp, "]\n" );

	fprintf(fp, "sf = [%g ", (float)sf[0]);
	for(i=1; i<6; i++)
		fprintf(fp, ",%g", (float)sf[i] );
	fprintf(fp, "]\n" );

	fprintf(fp, "xf = [%g ", (float)xf[0]);
	for(i=1; i<6; i++)
		fprintf(fp, ",%g", (float)xf[i] );
	fprintf(fp, "]\n" );

}

#include "rhmainu.c"
